/*
 *  Copyright (c) 2014 The CCP project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a Beijing Speedtong Information Technology Co.,Ltd license
 *  that can be found in the LICENSE file in the root of the web site.
 *
 *   http://www.yuntongxun.com
 *
 *  An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may
 *  be found in the AUTHORS file in the root of the source tree.
 */
#ifndef _HTTP_H_
#define _HTTP_H_

#include <string>
#include <map>

#define  HTTP_METHOD_UNKNOWN  -1
#define  HTTP_METHOD_GET      0
#define  HTTP_METHOD_PUT      1
#define  HTTP_METHOD_DELETE   2
#define  HTTP_METHOD_POST     3

#define  HTTP_CONTENT_DATA_ONLY_LEN "TRestClient(NoDataOnlyLenth)"

void SplitString(std::string& src , std::map<std::string,std::string> &param_map);

class THttp
{
protected:
    THttp();
    ~THttp();
public:    
    void  SetContentType(const char* contentType);
    void  SetContentData(const char* data,int len=-1);
	void  SetAgentData(const char* strAgentData);
	void  SetUserAgentData(const char* strUserAgentData);
	void  SetServerData(const char* strServerData);
	void  SetHost(const char* strHost);
	void  SetAccept(const char* strAccept);
	void  SetAuthenticateData(const char* strAuthenticate);
	void  SetAuthorizationData(const char* strAuthorization);
	void  SetMobileNumData(const char* strMobileNum);

    char* GetContentType();
    char* GetContentData();
	char* GetAgentData();
	char* GetUserAgentData();
	char* GetAuthorizationData();
	char* GetMobileNumData();
    int   GetContentLength();

protected:
    int   EncodeMessage(char* outputData,int& length);
    int   DecodeMessage(char* inputData,int length);

    char* m_strContentType;
    char* m_strContentData;
	char* m_strAgentData;
	char* m_strUserAgentData;
	char* m_strServerData;
	char* m_strHost;
	char* m_strAuthenticateData;
	char* m_strAuthorizationData;
	char* m_strMobileNumData;
	char* m_strAccept;
    int   m_nContentLen;
};
class THttpRequest : public THttp
{
public:
    THttpRequest();
    ~THttpRequest();

    void  SetMethod(int method);
    void  SetURI(const char* uri);
    
    int   GetMethod();
    char* GetURI();

    int   Encode(char* outputData,int& length);
    int   Decode(char* inputData,int length);
private:
    int   m_nMethod;
    char* m_strURI;
};

class THttpResponse : public THttp
{
public:
    THttpResponse();
    ~THttpResponse();

    void  SetStatusCode(int statusCode);
    int   GetStatusCode();

    int   Encode(char* outputData,int& length);
    int   Decode(char* inputData,int length);
private:
    int   m_nStatusCode;
};
#endif

