/*
 *  Copyright (c) 2014 The CCP project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a Beijing Speedtong Information Technology Co.,Ltd license
 *  that can be found in the LICENSE file in the root of the web site.
 *
 *   http://www.yuntongxun.com
 *
 *  An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may
 *  be found in the AUTHORS file in the root of the source tree.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "RestClient.h"
#include "base64.h"
#include "openssl/md5.h" 
#include "common.h"
#include "YYLog.h"
#include "tinyxml.h"

TRestClient::TRestClient(int bodyType,const char *p_restaddr, int restport, const char *p_mainaccount,  const char *p_mainaccountpwd,const char *p_subaccount,  const char *p_subaccountpwd, const char *p_voipid,const char *p_voippwd)
{
	memset(m_server,0,sizeof(m_server));
	//strncpy(m_server,p_restaddr,sizeof(m_server));
	//m_server[sizeof(m_server)-1]='\0';
	m_port=restport;

	strncpy(m_subaccount,p_subaccount,sizeof(m_subaccount));
	m_subaccount[sizeof(m_subaccount)-1]='\0';
	strncpy(m_subaccountpwd,p_subaccountpwd,sizeof(m_subaccountpwd));
	m_subaccountpwd[sizeof(m_subaccountpwd)-1]='\0';
	strncpy(m_voipid,p_voipid,sizeof(m_voipid));
	m_voipid[sizeof(m_voipid)-1]='\0';
	strncpy(m_voippwd,p_voippwd,sizeof(m_voippwd));
	m_voippwd[sizeof(m_voippwd)-1]='\0';

	strncpy(m_mainaccount,p_mainaccount,sizeof(m_mainaccount));
	m_mainaccount[sizeof(m_mainaccount)-1]='\0';
	strncpy(m_mainaccountpwd,p_mainaccountpwd,sizeof(m_mainaccountpwd));
	m_mainaccountpwd[sizeof(m_mainaccountpwd)-1]='\0';

	strncpy(m_RestVersion,"/2013-12-26",sizeof(m_RestVersion));//2013-12-26
	m_RestVersion[sizeof(m_RestVersion)-1]='\0';
	strncpy(m_https,"https://",sizeof(m_https));
	m_https[sizeof(m_https)-1]='\0';

	m_bUseSSL=true;
	if((bodyType>=REST_HTTP_BODY_TYPE_XML)&&(bodyType<=REST_HTTP_BODY_TYPE_JSON))
	{
		m_BodyType=bodyType;
	}else
	{
		m_BodyType=REST_HTTP_BODY_TYPE_XML;
	}

	//restַ httpֺ֧ͷhttpsݴ
	const char *p=NULL;
	p=strstr(p_restaddr,"http://");
	if(NULL!=p)
	{
		strncpy(m_server,p+7,sizeof(m_server));
		m_server[sizeof(m_server)-1]='\0';
		m_bUseSSL=false;
		strncpy(m_https,"http://",sizeof(m_https));
		m_https[sizeof(m_https)-1]='\0';
	}else
	{
		p=strstr(p_restaddr,"https://");
		if(NULL!=p)
		{
			strncpy(m_server,p+8,sizeof(m_server));
			m_server[sizeof(m_server)-1]='\0';
		}else
		{
			strncpy(m_server,p_restaddr,sizeof(m_server));
			m_server[sizeof(m_server)-1]='\0';
		}
	}
	if(m_bLogLevel>=LOG_INFO)
	{
		YYLog("RestClient","TRestClient::init serverǰ:%s,server:%s, port:%d, mainaccount:%s,subaccount:%s, voipid:%s",p_restaddr,m_server,m_port,m_mainaccount,m_subaccount,m_voipid);
	}
}
TRestClient::~TRestClient()
{
}
//ѯ˺
/*
  /{SoftVersion}/Accounts/{accountSid}/AccountInfo
  Ӧ
		 Լ	˵
statusCode	 String	 ѡ	 ״̬룬ȡֵ000000ɹ
dateCreated	 String	 ѡ	 ˻Ĵʱ
dateUpdated	 String	 ѡ	 ˻ʱ
friendlyName	 String	 ѡ	 ˻ƣΨһ64ӢĸͰɣĬΪĵַ
type	 String	 ѡ	 ˻ͣȡֵ0ã1עᣩĬֵΪ0
status	 String	 ѡ	 ˻״̬ȡֵ0δ12ͣ3رգĬֵΪ0
balance	 String	 ѡ	 ˻
*/
int TRestClient::synQueryAccountInfo(AccountInfo* outData)
{
	//ѡֶж,ַָ
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQueryAccountInfo, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}

	int ret=REST_OK;

	//ڴ
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;

	char uriStr[1024]={0};
	char auth64Str[100]={0};
	char mdStr[33]={0};  
	char body[1024]={0};
	char strHost[512]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));
		//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/AccountInfo?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_GET);

	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}

	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);

	//body
	
	outputData = new char[512*1024];
	int msglen = 0;
	if(request.Encode(outputData,msglen)<0)
	{
		YYLog("RestClient","TRestClient::synQueryAccountInfo,coding error,ret=%d",ret);
		ret=ERR_ENCODE_HTTP;
		goto CallBack;
	}

	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synQueryAccountInfo,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}

	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		YYLog("RestClient","TRestClient::synQueryAccountInfo,http code=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}

	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synQueryAccountInforest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQueryAccountInforest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQueryAccountInforest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("Account",pElementName))
		{
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if(!stricmp("friendlyName",pSubElementName))
				{
					strncpy(outData->friendlyName,pSubElementContent,sizeof(outData->friendlyName));
					outData->friendlyName[sizeof(outData->friendlyName)-1]='\0';

				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(outData->dateCreated,pSubElementContent,sizeof(outData->dateCreated));
					outData->dateCreated[sizeof(outData->dateCreated)-1]='\0';
				}else if(!stricmp("dateUpdated",pSubElementName))
				{
					strncpy(outData->dateUpdated,pSubElementContent,sizeof(outData->dateUpdated));
					outData->dateUpdated[sizeof(outData->dateUpdated)-1]='\0';
				}else if(!stricmp("type",pSubElementName))
				{
					outData->type=atoi(pSubElementContent);
				}else if(!stricmp("status",pSubElementName))
				{
					outData->status=atoi(pSubElementContent);
				}else if(!stricmp("balance",pSubElementName))
				{
					outData->balance=atof(pSubElementContent);
					//outData->balance=strtod(pSubElementContent,NULL);
					//if(m_bLogLevel>=LOG_DEBUG)
					//{
					//	YYLog("RestClient","TRestClient::synQueryAccountInfobalance=%lf,balan=%lf",atof(pSubElementContent),strtod(pSubElementContent,NULL));
					//}
				}

				pSubElement = pSubElement->NextSiblingElement(); 
			}
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synQueryAccountInfostatusCode=%d,statusMsg=%s,dateCreated=%s,dateUpdated=%s,friendlyName=%s,type=%d,status=%d,balance=%lf",outData->reason.statusCode,outData->reason.statusMsg,outData->dateCreated,outData->dateUpdated,outData->friendlyName,outData->type,outData->status,outData->balance);
	}
	return ret;
}
//˻
/*
appId	 String	 ѡ	 ӦId
friendlyName	 String	 ѡ	 ˻ơӢĸͰ˻ΨһƣƼʹõַ

*/
int TRestClient::synCreateSubAccount(SubAccountOne* outData,const char* appId,const char* friendlyName)
{
	//ѡֶж,ַָ
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synCreateSubAccount, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}

	if((NULL==friendlyName)||(NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synCreateSubAccount, ERR_NULL,%s","friendlyName,appId");}
		return ERR_NULL;
	}
	if((!stricmp("",friendlyName))||(!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synCreateSubAccount, ERR_NULL,friendlyName=%s,appId=%s",friendlyName,appId);}
		return ERR_NULL;
	}


	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;

	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[512]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));
	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/SubAccounts?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);

	//body
	char *dest = new char[512];
	int sub_len = unicode_to_utf8((char*)friendlyName,dest,512);//Ĵ

	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		if(sub_len<=0)
		{		
			sprintf(body,"%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><SubAccount><friendlyName>",friendlyName,"</friendlyName><appId>",appId,"</appId></SubAccount>");		
		}else
		{		
			sprintf(body,"%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><SubAccount><friendlyName>",dest,"</friendlyName><appId>",appId,"</appId></SubAccount>");	
		}
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		if(sub_len<=0)
		{		
			sprintf(body,"%s%s%s%s%s","{\"friendlyName\":\"",friendlyName,"\",\"appId\":\"",appId,"\"}");		
		}else
		{		
			sprintf(body,"%s%s%s%s%s","{\"friendlyName\":\"",dest,"\",\"appId\":\"",appId,"\"}");		
		}
	}
	delete[] dest;

	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synCreateSubAccount,coding error,ret=%d",ret);
		}
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synCreateSubAccount,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synCreateSubAccount,http code=%d",ret);
		}
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synCreateSubAccountrest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synCreateSubAccountrest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synCreateSubAccountrest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("SubAccount",pElementName))
		{
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if(!stricmp("subAccountSid",pSubElementName))
				{
					strncpy(outData->SubAccount.subAccountSid,pSubElementContent,sizeof(outData->SubAccount.subAccountSid));
					outData->SubAccount.subAccountSid[sizeof(outData->SubAccount.subAccountSid)-1]='\0';
				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(outData->SubAccount.dateCreated,pSubElementContent,sizeof(outData->SubAccount.dateCreated));
					outData->SubAccount.dateCreated[sizeof(outData->SubAccount.dateCreated)-1]='\0';
				}else if(!stricmp("subToken",pSubElementName))
				{
					strncpy(outData->SubAccount.subToken,pSubElementContent,sizeof(outData->SubAccount.subToken));
					outData->SubAccount.subToken[sizeof(outData->SubAccount.subToken)-1]='\0';
				}else if(!stricmp("voipAccount",pSubElementName))
				{
					strncpy(outData->SubAccount.voipAccount,pSubElementContent,sizeof(outData->SubAccount.voipAccount));
					outData->SubAccount.voipAccount[sizeof(outData->SubAccount.voipAccount)-1]='\0';
				}else if(!stricmp("voipPwd",pSubElementName))
				{
					strncpy(outData->SubAccount.voipPwd,pSubElementContent,sizeof(outData->SubAccount.voipPwd));
					outData->SubAccount.voipPwd[sizeof(outData->SubAccount.voipPwd)-1]='\0';
				}

				pSubElement = pSubElement->NextSiblingElement(); 
			}

			strncpy(outData->SubAccount.friendlyName,friendlyName,sizeof(outData->SubAccount.friendlyName));
			outData->SubAccount.friendlyName[sizeof(outData->SubAccount.friendlyName)-1]='\0';
			
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synCreateSubAccountstatusCode=%d,statusMsg=%s,dateCreated=%s,subAccountSid=%s,subToken=%s,voipAccount=%s,voipPwd=%s,friendlyName=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->SubAccount.dateCreated,outData->SubAccount.subAccountSid,outData->SubAccount.subToken,outData->SubAccount.voipAccount,outData->SubAccount.voipPwd,outData->SubAccount.friendlyName);
	}
	return ret;
}
//ѯ˻
/*

		 Լ	˵
appId	 String	 ѡ	 ӦId
startNo	 String	 ѡ	 ʼţĬϴ0ʼ
offset	 String	 ѡ	 һβѯС1100

*/
int TRestClient::synQuerySubAccounts(SubAccounts* outData,const char* appId,int startNo, int offset)
{
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQuerySubAccounts, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}
	if((NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQuerySubAccounts, ERR_NULL,%s","appId=NULL");}
		return ERR_NULL;
	}
	if((!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQuerySubAccounts ERR_NULL,appId=%s",appId);}
		return ERR_NULL;
	}

	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;

	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[512]={0};
	char *outputData=NULL;
	int restStatusCode;
	int count=0,m_count=0;

	outData->count=0;
	memset(outData->reason.statusMsg,0,sizeof(outData->reason.statusMsg));

	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/GetSubAccounts?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);

	//body
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		sprintf(body,"%s%d%s%d%s%s%s","<?xml version='1.0' encoding='utf-8'?><SubAccount><startNo>",startNo,"</startNo><offset>",offset,"</offset><appId>",appId,"</appId></SubAccount>");		
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		sprintf(body,"%s%d%s%d%s%s%s","{\"startNo\":\"",startNo,"\",\"offset\":\"",offset,"\",\"appId\":\"",appId,"\"}");		
	}
	
	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synQuerySubAccounts,coding error,ret=%d",ret);
		}
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synQuerySubAccounts,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synQuerySubAccounts,http code=%d",ret);
		}
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;
	TiXmlElement *pTmp= NULL; // ָʱ

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synQuerySubAccountsrest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQuerySubAccountsrest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQuerySubAccountsrest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}if(!stricmp("SubAccount",pElementName))
		{
			m_count++;
			if(1==m_count)
			{
				pTmp=pElement;//һбڵ
			}
		}else if(!stricmp("totalCount",pElementName))
		{
			count=atoi(pElementContent);
		}

		pElement = pElement->NextSiblingElement(); 
	}

	if(REST_OK!=ret)
	{
		goto CallBack;
	}
	if(m_count<count)
	{
		count=m_count;
	}
	if(count>0)
	{
		outData->count=count;
		SubAccountInfo* p=outData->SubAccounts;
		pElement=pTmp;
		if(offset<count)
		{
			count=offset;
		}
		for(int i=0;i<count;i++)
		{
			memset(p,0,sizeof(SubAccountInfo));
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if (!p)
				{
					break;
				}
				if(!stricmp("subAccountSid",pSubElementName))
				{
					strncpy(p->subAccountSid,pSubElementContent,sizeof(p->subAccountSid));
					p->subAccountSid[sizeof(p->subAccountSid)-1]='\0';
				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(p->dateCreated,pSubElementContent,sizeof(p->dateCreated));
					p->dateCreated[sizeof(p->dateCreated)-1]='\0';
				}else if(!stricmp("subToken",pSubElementName))
				{
					strncpy(p->subToken,pSubElementContent,sizeof(p->subToken));
					p->subToken[sizeof(p->subToken)-1]='\0';
				}else if(!stricmp("voipAccount",pSubElementName))
				{
					strncpy(p->voipAccount,pSubElementContent,sizeof(p->voipAccount));
					p->voipAccount[sizeof(p->voipAccount)-1]='\0';
				}else if(!stricmp("voipPwd",pSubElementName))
				{
					strncpy(p->voipPwd,pSubElementContent,sizeof(p->voipPwd));
					p->voipPwd[sizeof(p->voipPwd)-1]='\0';
				}else if(!stricmp("friendlyName",pSubElementName))
				{
					strncpy(p->friendlyName,pSubElementContent,sizeof(p->friendlyName));
					p->friendlyName[sizeof(p->friendlyName)-1]='\0';
				}

				pSubElement = pSubElement->NextSiblingElement(); 

			}//end while
			if(count==i+1)
			{
				break;
			}
			p++;
			pElement =pElement->NextSiblingElement("SubAccount"); //ڽڵ
			if (!pElement){
				break;
			}
		}//end for
	}




	//if(m_bLogLevel>=LOG_DEBUG)
	//{
	//	YYLog("RestClient","TRestClient::synQuerySubAccountsfriendlyName=%s,balance=%f",outData->friendlyName,outData->balance);
	//}
CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	return ret;
}
//ѯ˻
/*
appId	 String	 ѡ	 ӦId
friendlyName	 String	 ѡ	 ˻ơӢĸͰ˻ΨһƣƼʹõַ

*/
int TRestClient::synQuerySubAccountByName(SubAccountOne* outData,const char* appId,const char* friendlyName)
{
	//ѡֶж,ַָ
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQuerySubAccountByName, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}

	if((NULL==friendlyName)||(NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQuerySubAccountByName, ERR_NULL,%s","friendlyName,appId");}
		return ERR_NULL;
	}
	if((!stricmp("",friendlyName))||(!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synQuerySubAccountByName, ERR_NULL,friendlyName=%s,appId=%s",friendlyName,appId);}
		return ERR_NULL;
	}


	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;
	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[512]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));
	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/QuerySubAccountByName?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);

	//body
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{	
		sprintf(body,"%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><SubAccount><friendlyName>",friendlyName,"</friendlyName><appId>",appId,"</appId></SubAccount>");		
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		sprintf(body,"%s%s%s%s%s","{\"friendlyName\":\"",friendlyName,"\",\"appId\":\"",appId,"\"}");	
	}

	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synQuerySubAccountByName,coding error,ret=%d",ret);
		}
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synQuerySubAccountByName,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synQuerySubAccountByName,http code=%d",ret);
		}
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synQuerySubAccountByNamerest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQuerySubAccountByNamerest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQuerySubAccountByNamerest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("SubAccount",pElementName))
		{
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if(!stricmp("subAccountSid",pSubElementName))
				{
					strncpy(outData->SubAccount.subAccountSid,pSubElementContent,sizeof(outData->SubAccount.subAccountSid));
					outData->SubAccount.subAccountSid[sizeof(outData->SubAccount.subAccountSid)-1]='\0';
				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(outData->SubAccount.dateCreated,pSubElementContent,sizeof(outData->SubAccount.dateCreated));
					outData->SubAccount.dateCreated[sizeof(outData->SubAccount.dateCreated)-1]='\0';
				}else if(!stricmp("subToken",pSubElementName))
				{
					strncpy(outData->SubAccount.subToken,pSubElementContent,sizeof(outData->SubAccount.subToken));
					outData->SubAccount.subToken[sizeof(outData->SubAccount.subToken)-1]='\0';
				}else if(!stricmp("voipAccount",pSubElementName))
				{
					strncpy(outData->SubAccount.voipAccount,pSubElementContent,sizeof(outData->SubAccount.voipAccount));
					outData->SubAccount.voipAccount[sizeof(outData->SubAccount.voipAccount)-1]='\0';
				}else if(!stricmp("voipPwd",pSubElementName))
				{
					strncpy(outData->SubAccount.voipPwd,pSubElementContent,sizeof(outData->SubAccount.voipPwd));
					outData->SubAccount.voipPwd[sizeof(outData->SubAccount.voipPwd)-1]='\0';
				}

				pSubElement = pSubElement->NextSiblingElement(); 
			}//end while

			strncpy(outData->SubAccount.friendlyName,friendlyName,sizeof(outData->SubAccount.friendlyName));
			outData->SubAccount.friendlyName[sizeof(outData->SubAccount.friendlyName)-1]='\0';

		}

		pElement = pElement->NextSiblingElement(); 
	}//end while

CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synQuerySubAccountByNamestatusCode=%d,statusMsg=%s,dateCreated=%s,subAccountSid=%s,subToken=%s,voipAccount=%s,voipPwd=%s,friendlyName=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->SubAccount.dateCreated,outData->SubAccount.subAccountSid,outData->SubAccount.subToken,outData->SubAccount.voipAccount,outData->SubAccount.voipPwd,outData->SubAccount.friendlyName);
	}
	return ret;
}

//ģ
/*

		 Լ	˵
to	 String	 ѡ	 Žնֻ뼯ϣӢĶŷֿÿ͵ֻó100
appId	 String	 ѡ	 ӦId
templateId	 String	 ѡ	 ģId 
datas	 String	 ѡ	 ڵ
data	 String	 ѡ	 ݣ滻ģ{}

*/
int TRestClient::synSendTemplateSMS(SMSInfo* outData,const char* appId,const char* to,const char* templateId,const char** dataArr,int dataArrNum)
{
	//ѡֶж,ַָ
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synSendTemplateSMS, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}
	if((NULL==to)||(NULL==templateId)||(NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synSendTemplateSMS, ERR_NULL,%s","to,templateId,appId");}
		return ERR_NULL;
	}
	if((!stricmp("",to))||(!stricmp("",templateId))||(!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synSendTemplateSMS, ERR_NULL,to=%s,templateId=%s,appId=%s",to,templateId,appId);}
		return ERR_NULL;
	}

	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;
	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[512]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));
	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/SMS/TemplateSMS?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);

	//body
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		sprintf(body,"%s%s%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><TemplateSMS><to>",to,"</to><templateId>",templateId,"</templateId><appId>",appId,"</appId>");			
		
		if(NULL!=dataArr)
		{	
			strcat(body,"<datas>");
			char tmpSprintf[128]={0};
			for(int i=0;i<dataArrNum;i++)
			{
				if(NULL==dataArr[i])
				{
					ret= ERR_NULL;
					if(m_bLogLevel>=LOG_DEBUG)
					{YYLog("RestClient","TRestClient::synSendTemplateSMS,toArr[i] is ERR_NULL,i=%d,dataArr=%d",i,dataArr);}
					goto CallBack;
				}
				char *dest = new char[128];
				int sub_len = unicode_to_utf8((char*)dataArr[i],dest,128);//Ĵ
				if(sub_len>0)
				{
					sprintf(tmpSprintf,"%s%s%s","<data>",dest,"</data>");
				}
				else
				{
					sprintf(tmpSprintf,"%s%s%s","<data>",dataArr[i],"</data>");
				}
				delete[] dest;
				
				strcat(body,tmpSprintf);
			}
			strcat(body,"</datas>");
		}

		strcat(body,"</TemplateSMS>");

	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		sprintf(body,"%s%s%s%s%s%s%s","{\"to\":\"",to,"\",\"templateId\":\"",templateId,"\",\"appId\":\"",appId,"\"");			
		
		if(NULL!=dataArr)
		{	
			strcat(body,",\"datas\":[");
			char tmpSprintf[128]={0};
			for(int i=0;i<dataArrNum;i++)
			{
				if(NULL==dataArr[i])
				{
					ret= ERR_NULL;
					if(m_bLogLevel>=LOG_DEBUG)
					{YYLog("RestClient","TRestClient::synSendTemplateSMS,toArr[i] is ERR_NULL,i=%d,dataArr[i]=%d",i,dataArr[i]);}
					goto CallBack;
				}
				char *dest = new char[128];
				int sub_len = unicode_to_utf8((char*)dataArr[i],dest,128);//Ĵ
				if(sub_len>0)
				{
					sprintf(tmpSprintf,"%s%s%s","\"",dest,"\",");
				}
				else
				{
					sprintf(tmpSprintf,"%s%s%s","\"",dataArr[i],"\",");
				}
				delete[] dest;
				
				strcat(body,tmpSprintf);
			}
			//ȥһ
			int endindex=strlen(body)-1;
			body[endindex]='\0';
			strcat(body,"]");
		}
		strcat(body,"}");
	}


	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synSendTemplateSMS,coding error,ret=%d",ret);
		}
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synSendTemplateSMS,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synSendTemplateSMS,http code=%d",ret);
		}
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synSendTemplateSMSrest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synSendTemplateSMSrest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synSendTemplateSMSrest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("TemplateSMS",pElementName))
		{
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if(!stricmp("smsMessageSid",pSubElementName))
				{
					strncpy(outData->smsMessageSid,pSubElementContent,sizeof(outData->smsMessageSid));
					outData->smsMessageSid[sizeof(outData->smsMessageSid)-1]='\0';

				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(outData->dateCreated,pSubElementContent,sizeof(outData->dateCreated));
					outData->dateCreated[sizeof(outData->dateCreated)-1]='\0';
				}

				pSubElement = pSubElement->NextSiblingElement(); 
			}
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synSendTemplateSMSstatusCode=%d,statusMsg=%s,dateCreated=%s,smsMessageSid=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->dateCreated,outData->smsMessageSid);
	}
	return ret;
}


//֪ͨ
/*
appId 	String	ѡ	Ӧid	ff8080813bbcb2d6013b12361000b
to 	String	ѡ	к	008613811234567
mediaTxt	String	ѡ	ı	123
mediaName	String	ѡ	ļƣʽ wavmediaTxtͬʱΪաΪʱmediaTxtʧЧ
displayNum	String	ѡ	ʾ룬ʾȨɷ	
playTimes	String	ѡ	ѭŴ13ΣĬϲ1Ρ	
respUrl	    String	ѡ	ûк󣬷֪ͨӦòࡣ	
userData    String 	ѡ 	˽ݣ֪ͨ״̬֪ͨлȡ˲
txtSpeed    String 	ѡ 	ıתķٶȣȡֵΧ-500500mediaTxtЧЧ,ĬֵΪ0
txtVolume   String 	ѡ 	ıתСȡֵΧ-2020mediaTxtЧЧĬֵΪ0 
txtPitch    String 	ѡ 	ıתȡֵΧ-500500mediaTxtЧЧĬֵΪ0 
txtBgsound  String  ѡ    ıתıţĿǰͨѶƽ̨֧6ֱ16ֱ룬0ΪҪʱֵ֧Զ屳mediaTxtЧЧ 
playMode    String  ѡ    Ƿͬʱıļ , 0 1ǣĬ0Ȳı
*/
int TRestClient::synLandingCalls(CallInfo* outData,const char* appId,const char* to,const char* mediaName,const char* mediaTxt,int playTimes, const char* diaplayNum,const char* respUrl,const char* userData,const char* txtSpeed,const char* txtVolume,const char* txtPitch,const char* txtBgsound,const char* playMode)
{
	//POST /{SoftVersion}/Accounts/{accountSid}/Calls/LandingCalls
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synLandingCalls, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}
	if((NULL==to)||(NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synLandingCalls, ERR_NULL,%s","to=NULL,appId=NULL");}
		return ERR_NULL;
	}
	if((!stricmp("",to))||(!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synLandingCalls ERR_NULL,to=%s,appId=%s",to,appId);}
		return ERR_NULL;
	}

	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;
	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024]={0};
	char strHost[512]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));
	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/Calls/LandingCalls?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);

	//body
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		sprintf(body,"%s%s%s%d%s%s%s","<?xml version='1.0' encoding='utf-8'?><LandingCall><to>",to,"</to><playTimes>",playTimes,"</playTimes><appId>",appId,"</appId>");			
		if(NULL!=mediaName)
		{
			char strType[64]={0}; 			
			strcat(body,"<mediaName>");
			char *dest = new char[512];
			int sub_len = unicode_to_utf8((char*)mediaName,dest,512);//Ĵ
			if(sub_len>0)
			{
				strcat(body,dest);
			}
			else
			{
				strcat(body,mediaName);
			}
			delete[] dest;
			strcat(body,"</mediaName>");
		}
		if(NULL!=mediaTxt)
		{
			strcat(body,"<mediaTxt>");
			char *dest = new char[512];
			int sub_len = unicode_to_utf8((char*)mediaTxt,dest,512);//Ĵ
			if(sub_len>0)
			{		
				strcat(body,dest);
			}
			else
			{
				strcat(body,mediaTxt);
			}
			delete[] dest;
			strcat(body,"</mediaTxt>");
		}
		if(NULL!=diaplayNum)
		{
			strcat(body,"<displayNum>");
			strcat(body,diaplayNum);
			strcat(body,"</displayNum>");
		}
		if(NULL!=respUrl)
		{
			strcat(body,"<respUrl>");
			strcat(body,respUrl);
			strcat(body,"</respUrl>");
		}
		if(NULL!=userData)
		{
			strcat(body,"<userData>");
			strcat(body,respUrl);
			strcat(body,"</userData>");
		}
		if(NULL!=txtSpeed)
		{
			strcat(body,"<txtSpeed>");
			strcat(body,respUrl);
			strcat(body,"</txtSpeed>");
		}
		if(NULL!=txtVolume)
		{
			strcat(body,"<txtVolume>");
			strcat(body,respUrl);
			strcat(body,"</txtVolume>");
		}
		if(NULL!=txtPitch)
		{
			strcat(body,"<txtPitch>");
			strcat(body,respUrl);
			strcat(body,"</txtPitch>");
		}
		if(NULL!=txtBgsound)
		{
			strcat(body,"<txtBgsound>");
			strcat(body,respUrl);
			strcat(body,"</txtBgsound>");
		}
		if(NULL!=playMode)
		{
			strcat(body,"<playMode>");
			strcat(body,respUrl);
			strcat(body,"</playMode>");
		}
		strcat(body,"</LandingCall>");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		sprintf(body,"%s%s%s%d%s%s%s","{\"to\":\"",to,"\",\"playTimes\":\"",playTimes,"\",\"appId\":\"",appId,"\"");			
		if(NULL!=mediaName)
		{
			strcat(body,",\"mediaName\":\"");
			char *dest = new char[512];
			int sub_len = unicode_to_utf8((char*)mediaName,dest,512);//Ĵ
			if(sub_len>0)
			{
				strcat(body,dest);
			}
			else
			{
				strcat(body,mediaName);
			}
			delete[] dest;
			strcat(body,"\"");
		}
		if(NULL!=mediaTxt)
		{
			strcat(body,",\"mediaTxt\":\"");
			char *dest = new char[512];
			int sub_len = unicode_to_utf8((char*)mediaTxt,dest,512);//Ĵ
			if(sub_len>0)
			{		
				strcat(body,dest);
			}
			else
			{
				strcat(body,mediaTxt);
			}
			delete[] dest;
			strcat(body,"\"");
		}
		if(NULL!=diaplayNum)
		{
			strcat(body,",\"displayNum\":\"");
			strcat(body,diaplayNum);
			strcat(body,"\"");
		}
		if(NULL!=respUrl)
		{
			strcat(body,",\"respUrl\":\"");
			strcat(body,respUrl);
			strcat(body,"\"");
		}
		if(NULL!=userData)
		{
			strcat(body,",\"userData\":\"");
			strcat(body,userData);
			strcat(body,"\"");
		}
		if(NULL!=txtSpeed)
		{
			strcat(body,",\"txtSpeed\":\"");
			strcat(body,txtSpeed);
			strcat(body,"\"");
		}
		if(NULL!=txtVolume)
		{
			strcat(body,",\"txtVolume\":\"");
			strcat(body,txtVolume);
			strcat(body,"\"");
		}
		if(NULL!=txtPitch)
		{
			strcat(body,",\"txtPitch\":\"");
			strcat(body,txtPitch);
			strcat(body,"\"");
		}
		if(NULL!=txtBgsound)
		{
			strcat(body,",\"txtBgsound\":\"");
			strcat(body,txtBgsound);
			strcat(body,"\"");
		}
		if(NULL!=playMode)
		{
			strcat(body,",\"playMode\":\"");
			strcat(body,playMode);
			strcat(body,"\"");
		}
		strcat(body,"}");
	}
	

	request.SetContentData(body,strlen(body));


	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synLandingCalls,coding error,ret=%d",ret);
		}
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synLandingCalls,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		if(m_bLogLevel>=LOG_ERROR)
		{
		YYLog("RestClient","TRestClient::synLandingCalls,http code=%d",ret);
		}
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synLandingCallsrest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synLandingCallsrest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synLandingCallsrest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("LandingCall",pElementName))
		{
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if(!stricmp("callSid",pSubElementName))
				{
					strncpy(outData->callSid,pSubElementContent,sizeof(outData->callSid));
					outData->callSid[sizeof(outData->callSid)-1]='\0';

				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(outData->dateCreated,pSubElementContent,sizeof(outData->dateCreated));
					outData->dateCreated[sizeof(outData->dateCreated)-1]='\0';
				}

				pSubElement = pSubElement->NextSiblingElement(); 
			}
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synLandingCallsstatusCode=%d,statusMsg=%s,dateCreated=%s,callSid=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->dateCreated,outData->callSid);
	}
	return ret;
}


//֤
/*
appId	String	ѡ	ӦId  	12321321321321
verifyCode	String	ѡ	֤ݣΪֺӢĸִСд4-8λ 	123bs
playTimes	String	ѡ	Ŵ13 	1
to	String	ѡ	պ 	008613811975505
respUrl	String	ѡ	ûк󣬷֪ͨӦòࡣ	url
displayNum	String	ѡ	ʾ룬ʾȨɷ	01057234444

*/
int TRestClient::synVoiceVerifyCode(CallInfo* outData,const char* appId,const char* to, const char* verifyCode, int playTimes,const char* diaplayNum,const char* respUrl,const char* lang,const char* welcomePrompt,const char* playVerifyCode,const char* maxCallTime)
{
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synVoiceVerifyCode, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}
	if((NULL==to)||(NULL==appId)||(NULL==verifyCode))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synVoiceVerifyCode, ERR_NULL,%s","to=NULL,appId=NULL,verifyCode=NULL");}
		return ERR_NULL;
	}
	if((!stricmp("",to))||(!stricmp("",appId))||(!stricmp("",verifyCode)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synVoiceVerifyCode ERR_NULL,to=%s,appId=%s,verifyCode=%s",to,appId,verifyCode);}
		return ERR_NULL;
	}
	//POST  /{SoftVersion}/Accounts/{accountSid}/Calls/VoiceVerify
	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;

	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[5120]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));
	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/Calls/VoiceVerify?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);
	//body
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		sprintf(body,"%s%s%s%d%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><VoiceVerify><to>",to,"</to><playTimes>",playTimes,"</playTimes><appId>",appId,"</appId><verifyCode>",verifyCode,"</verifyCode>");			
		if(NULL!=diaplayNum)
		{
			strcat(body,"<displayNum>");
			strcat(body,diaplayNum);
			strcat(body,"</displayNum>");
		}
		if(NULL!=respUrl)
		{
			strcat(body,"<respUrl>");
			strcat(body,respUrl);
			strcat(body,"</respUrl>");
		}
			if(NULL!=lang)
		{
			strcat(body,"<lang>");
			strcat(body,lang);
			strcat(body,"</lang>");
		}
				if(NULL!=welcomePrompt)
		{
			strcat(body,"<welcomePrompt>");
			strcat(body,welcomePrompt);
			strcat(body,"</welcomePrompt>");
		}
					if(NULL!=playVerifyCode)
		{
			strcat(body,"<maxCallTime>");
			strcat(body,maxCallTime);
			strcat(body,"</maxCallTime>");
		}
		strcat(body,"</VoiceVerify>");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		sprintf(body,"%s%s%s%d%s%s%s%s%s","{\"to\":\"",to,"\",\"playTimes\":\"",playTimes,"\",\"appId\":\"",appId,"\",\"verifyCode\":\"",verifyCode,"\"");			
		if(NULL!=diaplayNum)
		{
			strcat(body,",\"displayNum\":\"");
			strcat(body,diaplayNum);
			strcat(body,"\"");
		}
		if(NULL!=respUrl)
		{
			strcat(body,",\"respUrl\":\"");
			strcat(body,respUrl);
			strcat(body,"\"");
		}
		if(NULL!=lang)
		{
			strcat(body,",\"lang\":\"");
			strcat(body,lang);
			strcat(body,"\"");
		}
		if(NULL!=welcomePrompt)
		{
			strcat(body,",\"welcomePrompt\":\"");
			strcat(body,welcomePrompt);
			strcat(body,"\"");
		}
		if(NULL!=playVerifyCode)
		{
			strcat(body,",\"playVerifyCode\":\"");
			strcat(body,playVerifyCode);
			strcat(body,"\"");
		}
		if(NULL!=maxCallTime)
		{
			strcat(body,",\"maxCallTime\":\"");
			strcat(body,maxCallTime);
			strcat(body,"\"");
		}
		strcat(body,"}");
	}
	
	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		YYLog("RestClient","TRestClient::synVoiceVerifyCode,coding error,ret=%d",ret);
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synVoiceVerifyCode,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		YYLog("RestClient","TRestClient::synVoiceVerifyCode,http code=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;

	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		//if(NULL==pElementContent) //ע͵<Account> ݻ᷵ؿ
		//{
		//	pElement = pElement->NextSiblingElement(); 
		//	continue;
		//}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synVoiceVerifyCoderest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synQueryAccountInforest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synVoiceVerifyCoderest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("VoiceVerify",pElementName))
		{
			pSubElement = pElement->FirstChildElement(); // ȡӽ
			while (pSubElement ) 
			{	
				pSubElementName=pSubElement->Value();
				pSubElementContent=pSubElement->GetText();
				if(NULL==pSubElementContent)
				{
					pSubElement = pSubElement->NextSiblingElement(); 
					continue;
				}
				if(!stricmp("callSid",pSubElementName))
				{
					strncpy(outData->callSid,pSubElementContent,sizeof(outData->callSid));
					outData->callSid[sizeof(outData->callSid)-1]='\0';

				}else if(!stricmp("dateCreated",pSubElementName))
				{
					strncpy(outData->dateCreated,pSubElementContent,sizeof(outData->dateCreated));
					outData->dateCreated[sizeof(outData->dateCreated)-1]='\0';
				}

				pSubElement = pSubElement->NextSiblingElement(); 
			}
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();

	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synVoiceVerifyCodestatusCode=%d,statusMsg=%s,dateCreated=%s,callSid=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->dateCreated,outData->callSid);
	}
	return ret;
}
//IVR
/*

	   	      Լ	  ˵
Appid	   String	 ѡ	 ӦId
number	   String	 ѡ	 к룬ΪDialڵ
userdata   String	 ѡ	 ûݣ<startservice>֪ͨзأֻдַΪDialڵ
record	   String	 ѡ	 Ƿ¼ΪtruefalseĬֵΪfalse¼ΪDialڵ
disnumber  String    ѡ    ûԺź룬ƽ̨ԺŹơ

*/
int TRestClient::synIvrDial(IVRCall* outData,const char* appId,const char* number, const char* userdata, bool record,const char* disnumber)
{
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synIvrDial, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}
	if((NULL==number)||(NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synIvrDial, ERR_NULL,%s","number=NULL,appId=NULL");}
		return ERR_NULL;
	}
	if((!stricmp("",number))||(!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synIvrDial ERR_NULL,to=%s,appId=%s",number,appId);}
		return ERR_NULL;
	}
	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;

	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[5120]={0};
	char *outputData=NULL;
	int restStatusCode;
	memset(outData,0,sizeof(*outData));

	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/ivr/dial?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	//if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	//{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	//}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	//{
	//	request.SetAccept("application/json");
	//	request.SetContentType("application/json;charset=utf-8");
	//}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);
	//body
	/*if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{*/
		sprintf(body,"%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><Request><Appid>",appId,"</Appid><Dial number=\"",number,"\"");			
		if(NULL!=userdata)
		{
			strcat(body," userdata=\"");
			strcat(body,userdata);
			strcat(body,"\"");
		}
		if(record)
		{
			strcat(body," record=\"true\"");
		}else
		{
			strcat(body," record=\"false\"");
		}
		if(NULL!=disnumber)
		{
			strcat(body," disnumber=\"");
			strcat(body,disnumber);
			strcat(body,"\"");
		}
		strcat(body,"></Dial></Request>");
	/*}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
	}*/
	
	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		YYLog("RestClient","TRestClient::synIvrDial,coding error,ret=%d",ret);
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synIvrDial,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		YYLog("RestClient","TRestClient::synIvrDial,http code=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		if(NULL==pElementContent) //
		{
			pElement = pElement->NextSiblingElement(); 
			continue;
		}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synIvrDialrest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synIvrDialrest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synIvrDialrest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("callSid",pElementName))
		{
			strncpy(outData->callSid,pElementContent,sizeof(outData->callSid));
			outData->callSid[sizeof(outData->callSid)-1]='\0';
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();

	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synIvrDialstatusCode=%d,statusMsg=%s,callSid=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->callSid);
	}
	return ret;
}
//ûصַ
/*
 
		 Լ	˵
appId	 String	 ѡ	 ӦId
date	 String	 ѡ	 day ǰһݣ00:00 C 23:59;weekǰһܵ(һ )monthʾһµݣϸ±ʾǰ¼1410ţѯ3·ݵݣ
keywords	 String	 ѡ	 ͻĲѯɿͻж岢ṩͨѶƽ̨ĬϲԴ˲

*/
int TRestClient::synGetBillRecordsURL(BillRecord* outData,const char* appId,const char* date, const char* keywords)
{
	if((NULL==outData))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synGetBillRecordsURL, ERR_NULL,%s","outData");}
		return ERR_NULL;
	}
	if((NULL==date)||(NULL==appId))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synGetBillRecordsURL, ERR_NULL,%s","date=NULL,appId=NULL");}
		return ERR_NULL;
	}
	if((!stricmp("",date))||(!stricmp("",appId)))
	{
		if(m_bLogLevel>=LOG_ERROR)
		{YYLog("RestClient","TRestClient::synGetBillRecordsURL ERR_NULL,date=%s,appId=%s",date,appId);}
		return ERR_NULL;
	}
	int ret=REST_OK;	
	THttpRequest request;
	THttpResponse response;
	THttpClient* m_HttpsClient=NULL;
	TiXmlDocument doc;

	char uriStr[1024]={0};
	char auth64Str[100];
	char mdStr[33]={0};  
	char body[1024];
	char strHost[5120]={0};
	char *outputData=NULL;
	int restStatusCode;

	memset(outData,0,sizeof(*outData));

	//ͷURL
	if(Md5AndBase64(m_mainaccount, m_mainaccountpwd,mdStr, auth64Str)!=REST_OK)
	{
		ret=ERR_MD5_BASE64;
		goto CallBack;
	}
	sprintf(uriStr,"%s%s:%d%s%s%s%s%s",m_https,m_server,m_port,m_RestVersion,"/Accounts/",m_mainaccount,"/BillRecords?sig=",mdStr);
	request.SetURI(uriStr);
	request.SetMethod(HTTP_METHOD_POST);
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		request.SetAccept("application/xml");
		request.SetContentType("application/xml;charset=utf-8");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		request.SetAccept("application/json");
		request.SetContentType("application/json;charset=utf-8");
	}
	request.SetAuthorizationData(auth64Str);
	sprintf(strHost,"%s:%d",m_server,m_port);
	request.SetHost(strHost);
	//body
	if(REST_HTTP_BODY_TYPE_XML==m_BodyType)
	{
		sprintf(body,"%s%s%s%s%s","<?xml version='1.0' encoding='utf-8'?><BillRecords><appId>",appId,"</appId><date>",date,"</date>");			
		if(NULL!=keywords)
		{
			strcat(body,"<keywords>");
			strcat(body,keywords);
			strcat(body,"</keywords>");
		}
		strcat(body,"</BillRecords>");
	}else if(REST_HTTP_BODY_TYPE_JSON==m_BodyType)
	{
		sprintf(body,"%s%s%s%s%s","{\"appId\":\"",appId,"\",\"date\":\"",date,"\"");			
		if(NULL!=keywords)
		{
			strcat(body,",\"keywords\":\"");
			strcat(body,keywords);
			strcat(body,"\"");
		}
		strcat(body,"}");
	}
	
	request.SetContentData(body,strlen(body));

	outputData = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		ret=ERR_ENCODE_HTTP;
		YYLog("RestClient","TRestClient::synGetBillRecordsURL,coding error,ret=%d",ret);
		goto CallBack;
	}
	m_HttpsClient = new THttpClient(m_server, m_port, m_bUseSSL);
	if(!m_HttpsClient->SynHttpRequestData(outputData,msglen,0,response))
	{
		ret= ERR_SYN_HTTP_REQUEST;
		YYLog("RestClient","TRestClient::synGetBillRecordsURL,ERR_SYN_HTTP_REQUEST, ret=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_REQUEST);
		goto CallBack;
	}
	//http response 
	if(200!=response.GetStatusCode())
	{
		ret=response.GetStatusCode();
		YYLog("RestClient","TRestClient::synGetBillRecordsURL,http code=%d",ret);
		sprintf(outData->reason.statusMsg,ERR_STR_SYN_HTTP_STATAUS_CODE);
		goto CallBack;
	}
	doc.Parse(response.GetContentData());
	TiXmlNode *pNode = NULL; // ָڵ
	TiXmlElement *pElement = NULL; // ָ
	TiXmlElement *pSubElement = NULL; // ָ

	const char *pElementName;
	const char *pElementContent;
	const char *pSubElementName;
	const char *pSubElementContent;

	pNode = doc.FirstChild("Response");		//ýĸ	
	if(!pNode){ 
		ret=ERR_XML_NODE;
		goto CallBack;
	}
	pElement =pNode->FirstChildElement(); 
	while (pElement) 
	{	
		pElementName=pElement->Value();
		pElementContent=pElement->GetText();
		if(NULL==pElementContent) 
		{
			pElement = pElement->NextSiblingElement(); 
			continue;
		}
		if(!stricmp("statusCode",pElementName))
		{
			restStatusCode=atoi(pElementContent);
			outData->reason.statusCode=restStatusCode;
			if(restStatusCode>0)
			{
				ret=restStatusCode;
				if(m_bLogLevel>=LOG_ERROR)
				{YYLog("RestClient","TRestClient::synGetBillRecordsURLrest code=%d,req={%s}",ret,outputData);}
				//goto CallBack;
			}
		}else if(!stricmp("statusMsg",pElementName))
		{
			strncpy(outData->reason.statusMsg,pElementContent,sizeof(outData->reason.statusMsg));
			outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			char *dest = new char[256];
			int sub_len = utf8_to_unicode(outData->reason.statusMsg,dest,256);
			if(sub_len>0)
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synGetBillRecordsURLrest codemsg=%s",dest);
				}
				strncpy(outData->reason.statusMsg,dest,sizeof(outData->reason.statusMsg));
				outData->reason.statusMsg[sizeof(outData->reason.statusMsg)-1]='\0';
			}
			else
			{
				if(m_bLogLevel>=LOG_ERROR)
				{
				YYLog("RestClient","TRestClient::synGetBillRecordsURLrest codemsg=%s",outData->reason.statusMsg);
				}
			}
			delete dest;
		}else if(!stricmp("downurl",pElementName))
		{
			strncpy(outData->downurl,pElementContent,sizeof(outData->downurl));
			outData->downurl[sizeof(outData->downurl)-1]='\0';		
		}else if(!stricmp("token",pElementName))
		{
			strncpy(outData->token,pElementContent,sizeof(outData->token));
			outData->token[sizeof(outData->token)-1]='\0';		
		}

		pElement = pElement->NextSiblingElement(); 
	}

CallBack:
	doc.Clear();
	if(NULL!=m_HttpsClient)
	{
		delete m_HttpsClient;
		m_HttpsClient=NULL;
	}
	if(NULL!=outputData)
	{
	   delete[] outputData;
	   outputData=NULL;
	}
	outData->reason.statusCode=ret;
	if(m_bLogLevel>=LOG_DEBUG)
	{
		YYLog("RestClient","TRestClient::synGetBillRecordsURLstatusCode=%d,statusMsg=%s,downurl=%s,token=%s",outData->reason.statusCode,outData->reason.statusMsg,outData->downurl,outData->token);
	}
	return ret;
}


//ߺ
int TRestClient::Md5AndBase64(const char* account, const char* accountpwd,char* mdrstr, char* base64str)
{
	int ret=REST_OK;
	if(!account||!accountpwd||!mdrstr||!mdrstr)
	{
		return ERR_NULL;
	}
	if((strlen(account)<=0)||(strlen(accountpwd)<=0))
	{
		if(m_bLogLevel>=LOG_DEBUG)
		{
			YYLog("RestClient","TRestClient::Md5AndBase64, ˺Ϊ account=%S,accountpwd=%s",account,accountpwd);
		}
		return ERR_NULL;
	}
	char dateStr[15];
	char sigStr[100];
	char authStr[100];
	unsigned char md[MD5_DIGEST_LENGTH]; 
	char* p=(char*)mdrstr;

	GetSecondTimeFmt_yyyyMMddHHmmss(dateStr);
	sprintf(sigStr,"%s%s%s",account,accountpwd,dateStr);
	MD5((const unsigned char*)sigStr, strlen(sigStr), md);
	for(int i=0; i<16; i++){  
        sprintf(p, "%02X", md[i]); 
		p+=2;
	}
	sprintf(authStr,"%s:%s",account,dateStr);
	Base64encode(base64str, authStr,strlen(authStr));
	return ret;
}
