/*
 *  Copyright (c) 2014 The CCP project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a Beijing Speedtong Information Technology Co.,Ltd license
 *  that can be found in the LICENSE file in the root of the web site.
 *
 *   http://www.yuntongxun.com
 *
 *  An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may
 *  be found in the AUTHORS file in the root of the source tree.
 */
#include "HttpClient.h"
#include "common.h"
#include "YYLog.h"
#include <memory.h>
#include <string.h>

#ifdef WIN32
#pragma warning(disable:4996)
#ifdef SUPPORT_SSL 
	#pragma comment(lib,"ssleay32.lib")
	#pragma comment(lib,"libeay32.lib")
	#pragma comment(lib,"ws2_32.lib")
#endif
#else
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#define closesocket close
#define strnicmp  strncasecmp

#endif

THttpClient::THttpClient(std::string server, int port, bool bUseSSL)
	:m_server(server),m_port(port),m_socket(-1),m_bUseSSL(bUseSSL),m_pTraceFun(NULL)
{
#ifdef WIN32
	static bool bInitSocket = false;
	if( !bInitSocket) 
	{
		bInitSocket = true;
		WORD wVersionRequested;
		WSADATA wsaData;
		wVersionRequested = MAKEWORD(2, 0);
		WSAStartup(wVersionRequested, &wsaData);
	}
#endif

#ifdef SUPPORT_SSL
	m_ssl_ctx = NULL;
	m_ssl = NULL;
#endif
	
}
THttpClient::~THttpClient()
{
	if(m_socket!=-1)
	{
		closesocket(m_socket);
	}
#ifdef SUPPORT_SSL
	if( m_ssl) {
		SSL_shutdown(m_ssl);
		SSL_free(m_ssl);
		SSL_CTX_free(m_ssl_ctx);
	}
#endif
}
void THttpClient::SetTraceFunc(TRACE_FUNC trace_func)
{
	m_pTraceFun = trace_func;
}

bool THttpClient::SynHttpRequest(THttpRequest& request,THttpResponse &response )
{
	m_errorMsg = "";
#ifdef SUPPORT_SSL
	if( m_bUseSSL && !m_ssl_ctx) 
	{
		SSL_library_init();
		SSL_load_error_strings();
		m_ssl_ctx = SSL_CTX_new(SSLv3_client_method());
		m_ssl =  SSL_new(m_ssl_ctx);
		SSL_CTX_set_verify(m_ssl_ctx, SSL_VERIFY_NONE, NULL);
	}
#else 
	if( m_bUseSSL)
		return false;
#endif

	char cport[32]={0};
	struct addrinfo *result = NULL;
	struct addrinfo hints;
	sprintf(cport,"%d",m_port);
	
	memset( &hints,0,sizeof(addrinfo));
	hints.ai_family = AF_INET;
	hints.ai_protocol=IPPROTO_TCP;
	hints.ai_socktype = SOCK_STREAM;
	struct hostent *p_iplist;
	if((m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0){
		m_errorMsg = "create socket error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequest,%s",m_errorMsg.c_str());}
		return false;
	}
	//dns
    if( getaddrinfo(m_server.c_str(),cport, NULL, &result) != 0) {
		m_errorMsg = "get address info error for " +m_server+":"+cport;
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequest,%s",m_errorMsg.c_str());}
		closesocket(m_socket);
		m_socket = -1;
		return false;
	}

	
	if(connect(m_socket, result->ai_addr, result->ai_addrlen) < 0){
		m_errorMsg = "connect to " +m_server+":"+cport+" error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequest,%s",m_errorMsg.c_str());}
		closesocket(m_socket);
		m_socket = -1;
		freeaddrinfo( result);
		return false;
	}
	freeaddrinfo( result);
#ifdef SUPPORT_SSL
	if( m_bUseSSL) 
	{
		SSL_set_fd(m_ssl,m_socket);
		if( SSL_connect(m_ssl) < 0 )
		{
			m_errorMsg = "ssl connect to " +m_server+":"+cport+" error";
			if(m_bLogLevel>=LOG_DEBUG)
			{YYLog("RestClient","THttpClient::SynHttpRequest,%s",m_errorMsg.c_str());}
			return false;
		}
	}
#endif
	if( sendHttpRequest(request)<0 ) {
		closesocket(m_socket);
		m_socket = -1;
		return false;
	}
	
	if( recvHttpResponse(response)<0) {
		closesocket(m_socket);
		m_socket = -1;
		return false;
	}
	return true;
}
bool THttpClient::AsynHttpRequest(THttpRequest& request,HttpResponseCallBack callback )
{
	return true;
}

int THttpClient::sendHttpRequest(THttpRequest& request)
{
	char *outputData = new char[512*1024];
	//char *outputDatatmp = new char[512*1024];
	int msglen = 0;
	if( request.Encode(outputData,msglen) < 0 )
	{
		m_errorMsg = " reqeust coding error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequest,%s",m_errorMsg.c_str());}
		delete outputData;
		return -1;
	}
	//Trace("[HttpClient][socket:%d] send message:\n%s\n\n",m_socket,outputData);
	if(m_bLogLevel>=LOG_PACKAGE)
	{
	YYLog("RestClient","THttpClient::sendHttpRequest,socket:%d send message:\n%s",m_socket,outputData);
	}
	int	sendlen = 0,len=0;
	while(sendlen <msglen )
	{
		if(!m_bUseSSL)
			//len = send(m_socket, outputData+sendlen, strlen(outputData)-sendlen, 0); 
			len = send(m_socket, outputData+sendlen, msglen-sendlen, 0);//add by zhanagjunliang 20130318 ¼ļ00ᵱַ
#ifdef SUPPORT_SSL
		else
			len = SSL_write(m_ssl,outputData+sendlen,strlen(outputData)-sendlen);
#endif		
		if(len <= 0){
			m_errorMsg = "send message eroor";
			if(m_bLogLevel>=LOG_DEBUG)
			{YYLog("RestClient","%s",m_errorMsg.c_str());}
			delete outputData;
			return -1;
		}
		sendlen += len;
	}
	delete outputData;
	return 0;
}
int THttpClient::recvHttpResponse(THttpResponse& response)
{
	int buflen = 1024*1024;
	const char *body_len_header = "Content-Length:";
	char *inputData = new char[buflen];
	char lenStr[64] = {0};
	int len = 0,recvlen=0, bodylen = -1;
	memset(inputData,0,buflen);
	char *body = NULL;
	char *unParsedHeader = inputData;
	do {
		if( !m_bUseSSL) 
			len = recv(m_socket, inputData+recvlen, buflen-recvlen, 0);
#ifdef SUPPORT_SSL
		else
			len = SSL_read(m_ssl,inputData+recvlen, buflen-recvlen);
#endif
		if( len <=0 )
			break;
		recvlen += len;
		if( bodylen < 0) 
		{
			char *header = strstr(unParsedHeader,"\r\n");
			while( header) 
			{
				if( strnicmp(header+2,body_len_header,strlen(body_len_header))) 
				{
					header = strstr(header+2,"\r\n");
					if( !header)
						break;
					unParsedHeader = header+2;
					continue;
				}

				char *endheader = strstr(header+2,"\r\n");
				if(endheader) {
					header += 2+strlen(body_len_header);
					strncpy( lenStr,header,endheader-header);
					bodylen = atoi(lenStr);
					break;
				}
			}

		} 
		// don't use else if here, because last if may change the bodylen
		if(bodylen >= 0 ) 
		{
			if(!body) {
				body = strstr(unParsedHeader-4,"\r\n\r\n");
				if( !body)
					unParsedHeader = inputData+recvlen;
				else 
					body += 4;
			}
			//if( body && (body-inputData+bodylen+1>= recvlen)) {
			if( body && (recvlen>=body-inputData+bodylen)) { //alter by zhangjunliang 20130318
				break;
			}
		}		
	} while(true);
	
	response.Decode(inputData,recvlen);
	
	char *dest = new char[buflen];
	int sub_len = utf8_to_unicode(inputData,dest,buflen);
	if(sub_len>0)
	{
		if(m_bLogLevel>=LOG_PACKAGE)
		{
		YYLog("RestClient","THttpClient::recvHttpResponse,socket=%d,recvlen=%d, receive message={%s}\r\n\r\n\r\n\r\n",m_socket,recvlen,dest);
		}
	}
	else
	{
		if(m_bLogLevel>=LOG_PACKAGE)
		{
		YYLog("RestClient","THttpClient::recvHttpResponse,socket=%d,recvlen=%d, receive message={%s}\r\n\r\n\r\n\r\n",m_socket,recvlen,inputData);
		}
	}
	delete dest;

	
	delete inputData;
	return 0;
}
bool THttpClient::SynHttpRequestData(char *data,int datalen ,int timeOUT,THttpResponse &response )
{
	m_errorMsg = "";
#ifdef SUPPORT_SSL
	if( m_bUseSSL && !m_ssl_ctx) 
	{
		SSL_library_init();
		SSL_load_error_strings();
		m_ssl_ctx = SSL_CTX_new(SSLv3_client_method());
		m_ssl =  SSL_new(m_ssl_ctx);
		SSL_CTX_set_verify(m_ssl_ctx, SSL_VERIFY_NONE, NULL);
	}
#else 
	if( m_bUseSSL)
		return false;
#endif

	char cport[32]={0};
	struct addrinfo *result = NULL;
	struct addrinfo hints;
	sprintf(cport,"%d",m_port);
	
	memset( &hints,0,sizeof(addrinfo));
	hints.ai_family = AF_INET;
	hints.ai_protocol=IPPROTO_TCP;
	hints.ai_socktype = SOCK_STREAM;
	struct hostent *p_iplist;
	if((m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0){
		m_errorMsg = "create socket error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequestData,%s",m_errorMsg.c_str());}
		return false;
	}
	//dns
    if( getaddrinfo(m_server.c_str(),cport, NULL, &result) != 0) {
		m_errorMsg = "get address info error for " +m_server+":"+cport;
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequestData,%s",m_errorMsg.c_str());}
		closesocket(m_socket);
		m_socket = -1;
		return false;
	}

	if(m_bLogLevel>=LOG_INFO)
	{
		m_errorMsg = "connect to " +m_server+":"+cport;
		YYLog("RestClient","THttpClient::SynHttpRequestData,%s,rest ip=%s,m_socket=%d",m_errorMsg.c_str(),inet_ntoa(((sockaddr_in*)(result->ai_addr))->sin_addr),m_socket);
	}


	if(connect(m_socket, result->ai_addr, result->ai_addrlen) < 0){
		m_errorMsg = "connect to " +m_server+":"+cport+" error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpRequestData,%s",m_errorMsg.c_str());}
		closesocket(m_socket);
		m_socket = -1;
		freeaddrinfo( result);
		return false;
	}
	//inet_ntoa(((sockaddr_in*)(result->ai_addr))->sin_addr);
	

	freeaddrinfo( result);
#ifdef SUPPORT_SSL
	if( m_bUseSSL) 
	{
	

		SSL_set_fd(m_ssl,m_socket);
		if( SSL_connect(m_ssl) < 0 )
		{
			closesocket(m_socket);	
			m_socket = -1;

			SSL_shutdown(m_ssl);
		    SSL_free(m_ssl);
		    SSL_CTX_free(m_ssl_ctx);
			m_ssl_ctx = NULL;
	        m_ssl = NULL;

			m_errorMsg = "ssl connect to " +m_server+":"+cport+" error";
			if(m_bLogLevel>=LOG_DEBUG)
			{YYLog("RestClient","THttpClient::SynHttpRequestData,%s",m_errorMsg.c_str());}
			return false;
		}
		if(timeOUT>0)
	    {
			unsigned long ul =1;
			int nRet;
			nRet = ioctlsocket(m_socket, FIONBIO, (unsigned long *) &ul);
			if (nRet == SOCKET_ERROR)
			{
				closesocket(m_socket);
				m_socket = -1;
				m_errorMsg = "SSL÷ʧܣ";
				if(m_bLogLevel>=LOG_DEBUG)
				{YYLog("RestClient","THttpClient::SynHttpRequestData,%s",m_errorMsg.c_str());}
				return false;
			}
		}
	}
#endif
	//
	if( sendHttpRequestData(data,datalen)<0 ) {
			closesocket(m_socket);
			m_socket = -1;
			return false;
		}
	//ճʱhttp ճʱѡ+selecthttpsճʱĵҪ+ʱʱhttps Ҫע봦
	if(timeOUT>0)//ʱ
	{

		if(!m_bUseSSL)//http
		{
			timeval waittime;
			waittime.tv_sec = timeOUT;
			waittime.tv_usec = 0;
			//int nStartTime = GetNowTime();
			uIn64 nStartTime =  GetTickCount();//
			fd_set ready;
			while(TRUE)
			{
				if(GetTickCount() - nStartTime >= timeOUT*1000)
				{
					closesocket(m_socket);
					m_socket = -1;
					m_errorMsg = "http select ʱ";
		            if(m_bLogLevel>=LOG_DEBUG)
					{YYLog("RestClient","THttpClient::SynHttpRequestData,%s",m_errorMsg.c_str());}
					return false;
				}
				  
				FD_ZERO(&ready);
				FD_SET(m_socket,&ready);
					
				if (select(0, &ready, NULL, NULL, &waittime) > 0)
				{
					if (FD_ISSET(m_socket, &ready))
					{
						
			          if( recvHttpResponse(response)<0) {
							closesocket(m_socket);
							m_socket = -1;
							return false;
						}
					
					}
				}
			}//end while	

		}else//https
		{
			if(recvHttpsTimeoutResponse(response,timeOUT)<0) {
				closesocket(m_socket);
				m_socket = -1;
				return false;
			}
		}

	}else//ǳʱ
	{
			if( recvHttpResponse(response)<0) {
				closesocket(m_socket);
				m_socket = -1;
				return false;
			}
	}
	return true;
}
bool THttpClient::SynHttpSetup(int family,int socktype,int protocol)
{
		m_errorMsg = "";
#ifdef SUPPORT_SSL
	if( m_bUseSSL && !m_ssl_ctx) 
	{
		SSL_library_init();
		SSL_load_error_strings();
		m_ssl_ctx = SSL_CTX_new(SSLv3_client_method());
		m_ssl =  SSL_new(m_ssl_ctx);
		SSL_CTX_set_verify(m_ssl_ctx, SSL_VERIFY_NONE, NULL);
	}
#else 
	if( m_bUseSSL)
		return false;
#endif

	char cport[32]={0};
	struct addrinfo *result = NULL;
	struct addrinfo hints;
	sprintf(cport,"%d",m_port);
	
	memset( &hints,0,sizeof(addrinfo));
	hints.ai_family = AF_INET;
	hints.ai_protocol=IPPROTO_TCP;
	hints.ai_socktype = SOCK_STREAM;

	struct hostent *p_iplist;
	//if((m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0){
	if((m_socket = socket(AF_INET, socktype, protocol)) < 0){
		m_errorMsg = "create socket error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpSetup,%s",m_errorMsg.c_str());}
		return false;
	}
	//dns
    if( getaddrinfo(m_server.c_str(),cport, NULL, &result) != 0) {
		m_errorMsg = "get address info error for " +m_server+":"+cport;
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpSetup,%s",m_errorMsg.c_str());}
		closesocket(m_socket);
		m_socket = -1;
		return false;
	}

	if(m_bLogLevel>=LOG_INFO)
	{
		m_errorMsg = "connect to " +m_server+":"+cport;
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpSetup,%s,rest ip=%s,m_socket=%d",m_errorMsg.c_str(),inet_ntoa(((sockaddr_in*)(result->ai_addr))->sin_addr),m_socket);}
	}

	if(connect(m_socket, result->ai_addr, result->ai_addrlen) < 0){
		m_errorMsg = "connect to " +m_server+":"+cport+" error";
		if(m_bLogLevel>=LOG_DEBUG)
		{YYLog("RestClient","THttpClient::SynHttpSetup,%s",m_errorMsg.c_str());}
		closesocket(m_socket);
		m_socket = -1;
		freeaddrinfo( result);
		return false;
	}
	freeaddrinfo( result);
#ifdef SUPPORT_SSL
	if( m_bUseSSL) 
	{
		SSL_set_fd(m_ssl,m_socket);
		if( SSL_connect(m_ssl) < 0 )
		{
			m_errorMsg = "ssl connect to " +m_server+":"+cport+" error";
			if(m_bLogLevel>=LOG_DEBUG)
			{YYLog("RestClient","THttpClient::SynHttpSetup,%s",m_errorMsg.c_str());}
			return false;
		}
	}
#endif

	return true;
}
int THttpClient::sendHttpRequestData(char *outputData,int msglen )
{

	char *dest = new char[msglen+1];
	int sub_len = utf8_to_unicode(outputData,dest,msglen+1);
	if(sub_len>0)
	{
		if((m_bLogLevel>=LOG_PACKAGE)&&(msglen<2048))
		{
			YYLog("RestClient","THttpClient::sendHttpRequestData,m_socket=%d,msglen=%d,outputData={%s}\r\n\r\n\r\n\r\n",m_socket,msglen,dest);
		}
	}
	else
	{
		if((m_bLogLevel>=LOG_PACKAGE)&&(msglen<2048))
		{
			YYLog("RestClient","THttpClient::sendHttpRequestData,m_socket=%d,msglen=%d,outputData={%s}\r\n\r\n\r\n\r\n",m_socket,msglen,outputData);
		}
	}
	delete dest;

	int	sendlen = 0,len=0,errnum=0,err=0;
	while(sendlen <msglen )
	{
		if(!m_bUseSSL)
			len = send(m_socket, outputData+sendlen, msglen-sendlen, 0);//add by zhanagjunliang 20130318 ¼ļ00ᵱַ
#ifdef SUPPORT_SSL
		else
			len = SSL_write(m_ssl,outputData+sendlen,msglen-sendlen);
#endif		
		if(len <= 0){
			if(!m_bUseSSL)
			{
				err=GetLastError();
				m_errorMsg = "send http message eroor";
				if(m_bLogLevel>=LOG_DEBUG)
				{YYLog("RestClient","THttpClient::sendHttpRequestData,len=%d,%s,Error=%d",len,m_errorMsg.c_str(),err);}
				if(WSAECONNRESET==err)
				{	
				  return err;
				}else
				{
					return -1;
				}
			}
#ifdef SUPPORT_SSL
			else
			{
				int nRes = SSL_get_error(m_ssl, len);
			    if (nRes == SSL_ERROR_WANT_WRITE)
				{
				continue;
				}
				else
				{
					m_errorMsg = "send https message eroor";
					if(m_bLogLevel>=LOG_DEBUG)
					{YYLog("RestClient","THttpClient::sendHttpRequestData,%s,err=%d",m_errorMsg.c_str(),nRes);}
                    return -1;
				}
			}
#endif
		}
		sendlen += len;
	}

	return 0;
}
int THttpClient::recvHttpsTimeoutResponse(THttpResponse& response,int timeOUT)
{
	if( !m_bUseSSL) 
		return -1;
	int buflen = 1024*1024;
	const char *body_len_header = "Content-Length:";
	char *inputData = new char[buflen];
	char lenStr[64] = {0};
	int len = 0,recvlen=0, bodylen = -1;
	memset(inputData,0,buflen);
	char *body = NULL;
	char *unParsedHeader = inputData;
	uIn64 nStartTime =  GetTickCount();//
	uIn64 NowTime= GetTickCount();//
	do {
		//if( !m_bUseSSL) 
		//	len = recv(m_socket, inputData+recvlen, buflen-recvlen, 0);
		NowTime=GetTickCount();
		if(NowTime - nStartTime >= timeOUT*1000)
		{
			delete inputData;
			m_errorMsg = "rev https message time out";
			if(m_bLogLevel>=LOG_DEBUG)
			{YYLog("RestClient","THttpClient::recvHttpsTimeoutResponse,%s,nStartTime=%.3fs,NowTime=%%.3fs,timeOUT=%d",m_errorMsg.c_str(),(double)nStartTime/1000,(double)NowTime/1000,timeOUT);}
			return -1;
		}
#ifdef SUPPORT_SSL
			len = SSL_read(m_ssl,inputData+recvlen, buflen-recvlen);
#endif
		if( len <=0 )
		{
			int nRes = SSL_get_error(m_ssl, len);
		    if (nRes ==  SSL_ERROR_WANT_READ)
			{
			continue;
			}
			else
			{
				delete inputData;
				m_errorMsg = "rev https message err";
			    if(m_bLogLevel>=LOG_DEBUG)
				{YYLog("RestClient","THttpClient::recvHttpsTimeoutResponse,%s,err=%d",m_errorMsg.c_str(),nRes);}
                return -1;
			}
		}
		recvlen += len;
		if( bodylen < 0) 
		{
			char *header = strstr(unParsedHeader,"\r\n");
			while( header) 
			{
				if( strnicmp(header+2,body_len_header,strlen(body_len_header))) 
				{
					header = strstr(header+2,"\r\n");
					if( !header)
						break;
					unParsedHeader = header+2;
					continue;
				}

				char *endheader = strstr(header+2,"\r\n");
				if(endheader) {
					header += 2+strlen(body_len_header);
					strncpy( lenStr,header,endheader-header);
					bodylen = atoi(lenStr);
					break;
				}
			}

		} 
		// don't use else if here, because last if may change the bodylen
		if(bodylen >= 0 ) 
		{
			if(!body) {
				body = strstr(unParsedHeader-4,"\r\n\r\n");
				if( !body)
					unParsedHeader = inputData+recvlen;
				else 
					body += 4;
			}
			//if( body && (body-inputData+bodylen+1>= recvlen)) {
			if( body && (body-inputData+bodylen<= recvlen)) { //alter by zhangjunliang 20130318
				break;
			}
		}		
	} while(true);
	response.Decode(inputData,recvlen);

	//Trace("[HttpClient][socket:%d] receive message:\n%s\n\n",m_socket,inputData);
	
	//char *dest = new char[buflen];
	//int sub_len = utf8_to_unicode(inputData,dest,buflen);
	//if(sub_len>0)
	//{
	//	//Trace("[HttpClient][socket:%d] receive message:\n%s\n\n",m_socket,dest);
	//	//YYLog("RestClient","THttpClient::recvHttpResponse,socket:%d receive message:\n%s",m_socket,dest);
	//}
	//else
	//{
	//	//Trace("[HttpClient][socket:%d] receive message:\n%s\n\n",m_socket,inputData);
	//	//YYLog("RestClient","THttpClient::recvHttpResponse,socket:%d receive message:\n%s",m_socket,inputData);
	//}
	//delete dest;

	delete inputData;
	return 0;
}
int THttpClient::recvHttpResponseData(char *inputData,int buflen)
{

	int len = 0,recvlen=0;
	do {
		if( !m_bUseSSL) 
			len = recv(m_socket, inputData+recvlen, buflen-recvlen, 0);
#ifdef SUPPORT_SSL
		else
			len = SSL_read(m_ssl,inputData+recvlen, buflen-recvlen);
#endif
		if((m_bLogLevel>=LOG_PACKAGE))//&&(len<2048)
		{
			YYLog("RestClient","THttpClient::recvHttpResponseData,socket=%d, len=%d,inputData={%s}\r\n\r\n\r\n\r\n",m_socket,len,inputData);
		}
		if(len<0)
		{
			if(m_bLogLevel>=LOG_PACKAGE)
		   {
			YYLog("RestClient","TRestClient::THttpClient::recvHttpResponseData,Error=%d",GetLastError());	
			//WSAEMSGSIZE
				//ݱ̫޷ȫװ뻺ʱСbuflenҪinputDataСһճȣеֵڶʧ
			}
		
		}
		if( len <=0 )
			break;
		recvlen += len;
		
	} while(true);

	
	//Trace("[HttpClient][socket:%d] receive message:\n%s\n\n",m_socket,inputData);
	
	//char *dest = new char[buflen];
	//int sub_len = utf8_to_unicode(inputData,dest,buflen);
	//if(sub_len>0)
	//{
	//	//Trace("[HttpClient][socket:%d] receive message:\n%s\n\n",m_socket,dest);
	//	//YYLog("RestClient","THttpClient::recvHttpResponse,socket:%d receive message:\n%s",m_socket,dest);
	//}
	//else
	//{
	//	//Trace("[HttpClient][socket:%d] receive message:\n%s\n\n",m_socket,inputData);
	//	//YYLog("RestClient","THttpClient::recvHttpResponse,socket:%d receive message:\n%s",m_socket,inputData);
	//}
	//delete dest;

	
	return recvlen;
}
void  THttpClient::Trace(const char*fmt,...)
{
	va_list args;
    va_start(args, fmt);
    if( !m_pTraceFun) 
		vprintf(fmt,args);
	else
		m_pTraceFun(fmt, args);
    va_end(args);
}
