/*
 *  Copyright (c) 2014 The CCP project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a Beijing Speedtong Information Technology Co.,Ltd license
 *  that can be found in the LICENSE file in the root of the web site.
 *
 *   http://www.yuntongxun.com
 *
 *  An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may
 *  be found in the AUTHORS file in the root of the source tree.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include "http.h"
#include "YYLog.h"

#pragma warning(disable : 4996)
#define ASSERT assert

#ifndef WIN32
#define strnicmp  strncasecmp
#define stricmp   strcasecmp
#endif

#define COPYSTR(dest,sour) if(dest) delete dest;if(sour){dest=new char[strlen(sour)+1];strcpy(dest,sour);}
static void TrimAll(char* sour)
{
   char* p=sour;
   if(*p==0)
      return;
   
   for(;;)
   {
      if((unsigned)*p >0x20 || *p ==0)
         break;
      p++;
   }
   strcpy(sour,p);
   
   if(*p ==0)
      return;
   
   p=sour+strlen(sour);
   for(;;)
   {
      if((unsigned)*p >0x20)
      {
         *(p+1)=0;
         return;
      }
      p--;
   }
}
void SplitString(std::string& src , std::map<std::string,std::string> &param_map)
{
	std::string key,value;
	size_t start = 0;
	do {
		size_t pos = src.find_first_of("=",start);
		if( pos == std::string::npos) {
			break;
		}
		key = src.substr(start,pos-start);
		start = pos+1;
		pos = src.find_first_of("&",start);
		if( pos == std::string::npos) {
			value=src.substr(start);
			param_map[key] = value;
			break;
		}
		value = src.substr(start,pos-start);
		param_map[key] = value;
		start = pos+1;
	}while(true);
}

THttp::THttp()
{
    m_strContentType=NULL;
    m_strContentData=NULL;
	m_strAgentData = NULL;
	m_strUserAgentData = NULL;
	m_strServerData = NULL;
	m_strHost = NULL;
	m_strAuthenticateData = NULL; 
	m_strAuthorizationData = NULL;
	m_strMobileNumData = NULL;
	m_strAccept = NULL;
    m_nContentLen =-1;
}

THttp::~THttp()
{
    if(m_strContentType)
        delete m_strContentType;

    if(m_strContentData)
        delete m_strContentData;

	if(m_strAgentData)
        delete m_strAgentData;

	if(m_strUserAgentData)
        delete m_strUserAgentData;

	if(m_strServerData)
        delete m_strServerData;

	if(m_strHost)
        delete m_strHost;

	if(m_strAuthenticateData)
        delete m_strAuthenticateData;

	if(m_strAuthorizationData)
        delete m_strAuthorizationData;

	if(m_strAccept)
		delete m_strAccept;

	if(m_strMobileNumData)
        delete m_strMobileNumData;
}

char* THttp::GetContentType()
{
    return m_strContentType;
}

char* THttp::GetContentData()
{
    return m_strContentData;
}

char* THttp::GetAgentData()
{
    return m_strAgentData;
}

char* THttp::GetUserAgentData()
{
    return m_strUserAgentData;
}

char* THttp::GetAuthorizationData()
{
    return m_strAuthorizationData;
}

char* THttp::GetMobileNumData()
{
    return m_strMobileNumData;
}

int THttp::GetContentLength()
{
    return m_nContentLen;
}

void  THttp::SetContentType(const char* contentType)
{
    ASSERT(contentType);

    COPYSTR(m_strContentType,contentType);

    if(m_strContentType)
    {
        TrimAll(m_strContentType);
    }
}

void  THttp::SetContentData(const char* data,int len)
{
    ASSERT(data);

    //printf("the test len[%d]\n", len);
    if(len == -1)
    {
        len = strlen(data);
    }

    
    if(m_strContentData)
    {
        delete m_strContentData;
    }

	m_nContentLen =len;

	if (len < 1)
	{
		return;
	}
	if(!stricmp(HTTP_CONTENT_DATA_ONLY_LEN,data))//Դϴأȱݳȣִμ
    {
		len = strlen(data);
        m_strContentData = new char[len+1];//+1 ԭ len=10Ϊ0-10
		memset(m_strContentData,0,len+1);
        memcpy(m_strContentData,data,len);
		m_strContentData[len]='\0';;
	}else
	{
    m_strContentData = new char[len+1];
	memset(m_strContentData,0,len+1);
    memcpy(m_strContentData,data,len);
    m_strContentData[len]='\0';;
	}
	//if(m_bLogLevel>=LOG_DEBUG)
	//{
	//	YYLog("RestClient","THttp::SetContentData m_strContentData=%s",m_strContentData);
	//}
   //printf("the test \n");
}



void  THttp::SetAgentData(const char* strAgentData)
{
    ASSERT(strAgentData);
	
    COPYSTR(m_strAgentData,strAgentData);
	
    if(m_strAgentData)
    {
        TrimAll(m_strAgentData);
    }
}


void  THttp::SetUserAgentData(const char* strUserAgentData)
{
    ASSERT(strUserAgentData);
	
    COPYSTR(m_strUserAgentData,strUserAgentData);
	
    if(m_strUserAgentData)
    {
        TrimAll(m_strUserAgentData);
    }
}

void  THttp::SetServerData(const char* strServerData)
{
    ASSERT(strServerData);
	
    COPYSTR(m_strServerData,strServerData);
	
    if(m_strServerData)
    {
        TrimAll(m_strServerData);
    }
}


void  THttp::SetHost(const char* strHost)
{
    ASSERT(strHost);
	
    COPYSTR(m_strHost,strHost);
	
    if(m_strHost)
    {
        TrimAll(m_strHost);
    }
}


void  THttp::SetAuthenticateData(const char* strAuthenticateData)
{
    ASSERT(strAuthenticateData);
	
    COPYSTR(m_strAuthenticateData,strAuthenticateData);
	
    if(m_strAuthenticateData)
    {
        TrimAll(m_strAuthenticateData);
    }
}


void  THttp::SetAuthorizationData(const char* strAuthorizaionData)
{
    ASSERT(strAuthorizaionData);
	
    COPYSTR(m_strAuthorizationData,strAuthorizaionData);
	
    if(m_strAuthorizationData)
    {
        TrimAll(m_strAuthorizationData);
    }
}


void  THttp::SetMobileNumData(const char* strMobileNumData)
{
    ASSERT(strMobileNumData);
	
    COPYSTR(m_strMobileNumData,strMobileNumData);
	
    if(m_strMobileNumData)
    {
        TrimAll(m_strMobileNumData);
    }
}

void  THttp::SetAccept(const char* strAccept)
{
    ASSERT(strAccept);
	
    COPYSTR(m_strAccept,strAccept);
	
    if(m_strAccept)
    {
        TrimAll(m_strAccept);
    }
}

int THttp::EncodeMessage(char* outputData,int& length)
{
    //set date
    char strDate[64]={0};
    time_t lt = time(NULL);
    strcpy(strDate,asctime(localtime(&lt)));
    strDate[strlen(strDate)-1]=0;

    strcat(outputData,"Date:");
    strcat(outputData,strDate);
    strcat(outputData,"\r\n");

	if(m_strAccept) 
	{
		strcat(outputData,"Accept:");
		strcat(outputData,m_strAccept);
		strcat(outputData,"\r\n");
    }
	if(m_strAuthenticateData)
	{
		strcat(outputData,"WWW-Authenticate:");
		strcat(outputData,m_strAuthenticateData);
		strcat(outputData,"\r\n");
    }
	if( m_strAuthorizationData) 
	{
		strcat(outputData,"Authorization:");
		strcat(outputData,m_strAuthorizationData);
		strcat(outputData,"\r\n");
	}
	if(m_strHost)
	{
		strcat(outputData,"Host:");
		strcat(outputData,m_strHost);
		strcat(outputData,"\r\n");
    }
	if(m_strAgentData)
	{
		strcat(outputData,"Agent:");
		strcat(outputData,m_strAgentData);
		strcat(outputData,"\r\n");
    }
	if(m_strServerData)
	{
		strcat(outputData,"Server:");
		strcat(outputData,m_strServerData);
		strcat(outputData,"\r\n");
    }
    //set connection
    strcat(outputData,"Connection:close\r\n");

    //set content type
	 if(m_strContentType)//ȺΪҲҪͣ415
    {
        strcat(outputData,"Content-Type:");
        strcat(outputData,m_strContentType);
        strcat(outputData,"\r\n");
    }
    else
    {
        strcat(outputData,"Content-Type:text/plain\r\n");
    }
    if(m_strContentData)
    {

  /*      if(m_strContentType)
        {
            strcat(outputData,"Content-Type:");
            strcat(outputData,m_strContentType);
            strcat(outputData,"\r\n");
        }
        else
        {
            strcat(outputData,"Content-Type:text/plain\r\n");
        }*/
        char lengthHeader[256];
        sprintf(lengthHeader,"Content-Length:%d\r\n",m_nContentLen);
        strcat(outputData,lengthHeader);
    }
    else
    {
        strcat(outputData,"Content-Length:0\r\n");
    }
    strcat(outputData,"\r\n");

    length=strlen(outputData);
    if(m_strContentData)
    {
		if(!stricmp(HTTP_CONTENT_DATA_ONLY_LEN,m_strContentData))//Դϴأȱݳȣִμ
		{
         return 0;
		}else
		{
        memcpy(outputData+length,m_strContentData,m_nContentLen);
        length += m_nContentLen;
		}
    }
    return 0;
}
int THttp::DecodeMessage(char* inputData,int length)
{
    char*  body=strstr(inputData,"\r\n\r\n");
    if(body ==NULL)
        return -1;
    body+=4;

    if(body -  inputData >  10240)
    {
    	return -1;
    }
    char header[10240];
    int  count=0;
    for(int i=0;i<length;i++)
    {
        if(inputData[i] == '\r')
        {
            if(strncmp(inputData+i,"\r\n",2)==0)
            {
                header[count]=0;
            
                if(strnicmp(header,"content-type:",13)==0)
                {
                    SetContentType(header+13);
                }
				else if(strnicmp(header,"Agent:",6)==0)
                {
                    SetAgentData(header+6);
                }
				else if(strnicmp(header,"User-Agent:",11)==0)
                {
                    SetUserAgentData(header+11);
                }
				else if(strnicmp(header,"Authorization:",14)==0)
                {
                    SetAuthorizationData(header+14);
                }
				else if(strnicmp(header,"X-Up-Calling-Line-ID:",21)==0)
                {
                    SetMobileNumData(header+21);
                }
                else if(strnicmp(header,"content-length:",15)==0)
                {
					m_nContentLen = atoi(header+15);
//                     if(atoi(header+15) != (int)strlen(body))
//                     {
//                         return -2;
//                     }
                }
            }
            if(strncmp(inputData+i,"\r\n\r\n",4)!=0)
            {
                count=0;
                i+=1;
                continue;
            }
            break;
        }
        header[count++]=inputData[i];
    }

    SetContentData(body,m_nContentLen);
    return 0;
}

THttpRequest::THttpRequest()
{
    m_nMethod=HTTP_METHOD_UNKNOWN;
    m_strURI=NULL;
}

THttpRequest::~THttpRequest()
{
    if(m_strURI)
        delete m_strURI;
}

void  THttpRequest::SetMethod(const int method)
{
    ASSERT(method == HTTP_METHOD_GET || method == HTTP_METHOD_PUT || method==HTTP_METHOD_DELETE|| method==HTTP_METHOD_POST);
    m_nMethod=method;
}

void  THttpRequest::SetURI(const char* uri)
{
    ASSERT(uri);

    COPYSTR(m_strURI,uri);
}

int  THttpRequest::GetMethod()
{
    return m_nMethod;
}

char* THttpRequest::GetURI()
{
    return m_strURI;
}

int  THttpRequest::Encode(char* outputData,int& length)
{
    static const char* MethodName[4]={"GET","PUT","DELETE","POST"};

    if(m_nMethod == HTTP_METHOD_UNKNOWN)
        return -1;

    if(m_strURI == NULL)
        return -2;

    //set request line
    sprintf(outputData,"%s %s HTTP/1.1\r\n",MethodName[m_nMethod],m_strURI);

    //set body
    EncodeMessage(outputData,length);

    outputData[length]='\0';
    return 0;
}

int  THttpRequest::Decode(char* inputData,int length)
{
    if(inputData ==NULL)
        return -1;

    int    method;
    char   uri[1024]="";

    char strType[1024];
    char strVersion[1024]="";

    char * strend = NULL;
    strend = strstr(inputData, "\r\n");

    if(strend-inputData >=512)
		return -1;
   	
    sscanf(inputData,"%s %s %s",strType,uri,strVersion);
    if(strcmp(strType,"GET") ==0)
    {
        method=HTTP_METHOD_GET;
    }
    else if(strcmp(strType,"PUT") ==0)
    {
        method=HTTP_METHOD_PUT;
    }
    else if(strcmp(strType,"DELETE") ==0)
    {
        method=HTTP_METHOD_DELETE;
    }
	else if(strcmp(strType,"POST") ==0)
    {
        method=HTTP_METHOD_POST;
    }
    else
    {
        return -3;
    }

//     if(strcmp(strVersion,"HTTP/1.1") !=0)  //ֻϢWAPغHTTPϢİ汾ʱΪ1.0
//         return -4;
   
 
    if(DecodeMessage(inputData,length) !=0)
        return -5;

    SetMethod(method);
    SetURI(uri);
    return 0;
}


struct TResponseStatus
{
    int  code;
    const char* desc;
};

const TResponseStatus responseStats[] = \
{
  /* Informational 1xx */
  {100, "Continue"},
  {101, "Switching Protocols"},

  /* Successful 2xx */
  {200, "OK"},
  {201, "Created"},
  {202, "Accepted"},
  {203, "Non-Authoritative Information"},
  {204, "No Content"},
  {205, "Reset Content"},
  {206, "Partial Content"},

  /* Redirection 3xx */
  {300, "Multiple Choices"},
  {301, "Moved Permanently"},
  {302, "Found"},
  {303, "See Other"},
  {304, "Not Modified"},
  {305, "Use Proxy"},
  // 306 Unused
  {307, "Temporary Redirect"},
 
  /* Client Error 4xx */
  {400, "Bad Request"},
  {401, "Unauthorized"},
  {402, "Payment Required"},
  {403, "Forbidden"},
  {404, "Not Found"},
  {405, "Method Not Allowed"},
  {406, "Not Acceptable"},
  {407, "Proxy Authentication Required"},
  {408, "Request Timeout"},
  {409, "Conflict"},
  {410, "Gone"},
  {411, "Length Required"},
  {412, "Precondition Failed"},
  {413, "Request Entity Too Large"},
  {414, "Request-URI Too Long"},
  {415, "Unsupported Media Type"},
  {416, "Requested Range Not Satisfiable"},
  {417, "Expectation Failed"},

  /* Server Error 5xx */
  {500, "Internal Server Error"},
 // {501, "Not Implemented"},
  {501, "Billing Failed"},
  {502, "Bad Gateway"},
  {503, "Service Unavailable"},
  {504, "Gateway Timeout"},
  {505, "HTTP Version Not Supported"},
  {511, "Illegal User Status"},
  {512, "Exceed Max Downloads Limit"},
  {513, "Exceed Max Billing Money this Period"}
};

const char* getStatusDescription(int code)
{
    for (int i=0; i < sizeof(responseStats) / sizeof(responseStats[0]); i++)
    {
        if (responseStats[i].code == code)
        {
            return responseStats[i].desc; 
        }
    }
    return NULL;
}

THttpResponse::THttpResponse()
{
    m_nStatusCode=0;
}

THttpResponse::~THttpResponse()
{
    
}

int  THttpResponse::Encode(char* outputData,int& length)
{
    const char* desc=getStatusDescription(m_nStatusCode);
    if(desc ==NULL)
        return -1;

    sprintf(outputData,"HTTP/1.1 %d %s\r\n",m_nStatusCode,desc);
    EncodeMessage(outputData,length);
    outputData[length]=0;
    return 0;
}

int  THttpResponse::Decode(char* inputData,int length)
{
    if(inputData ==NULL)
        return -1;
    
    char strVersion[256]={0};
    int  statusCode=0;
    char strDesc[256]={0};
	sscanf(inputData,"%s %d %s\r\n",strVersion,&statusCode,strDesc);
    
//     if(strcmp(strVersion,"HTTP/1.1") !=0)   //ֻϢWAPغHTTPϢİ汾ʱΪ1.0
//         return -2;

    const char* t=getStatusDescription(statusCode);
    if(t ==NULL)
        return -3;

//     if(strcmp(strDesc,t)!=0)
//         return -4;

    if(!strstr(inputData,"\r\n\r\n"))
        return -5;

    if(DecodeMessage(inputData,length) !=0)
        return -6;

    SetStatusCode(statusCode);
    return 0;
}

void THttpResponse::SetStatusCode(int statusCode)
{
    m_nStatusCode=statusCode;
}
int THttpResponse::GetStatusCode()
{
    return m_nStatusCode;
}
